@@ -1,8 +1,8 @@
 ;;; honest-report.el --- make bug report with screenshot and keylog
 
-;; Copyright (c) 2005, 2006
-;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;; $Id: honest-report.el,v 1.5 2006/03/22 13:29:25 hira Exp $
+;; Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012
+;;   HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
+;; $Id: honest-report.el,v 1.13 2011-12-31 15:07:29 hira Exp $
 ;;
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -43,15 +43,15 @@
         (msg (honest-report-message))
         (scr (honest-report-screenshot)))
     (honest-report-setup)
-    (mapcar (lambda (a) (apply #'honest-report-insert a))
-            `(
-              ("Header"          ,header)
-              ("Emacs version"   ,ver)
-              ("Recent keys"     ,key)
-              ("Recent messages" ,msg)
-              ("Screen shot"     ,scr)
-              ("Footer"          ,footer)
-              ))
+    (mapc (lambda (a) (apply #'honest-report-insert a))
+          `(
+            ("Header"          ,header)
+            ("Emacs version"   ,ver)
+            ("Recent keys"     ,key)
+            ("Recent messages" ,msg)
+            ("Screen shot"     ,scr)
+            ("Footer"          ,footer)
+            ))
     (goto-char (point-max))))
 
 (defun honest-report-insert (title content)
@@ -113,13 +113,24 @@ which lack `window-list'."
             ("Emacs" . ,(format "%s (%s) of %s"
                               emacs-version
                               system-configuration
-                              (howm-emacs-build-time)))
+                              (honest-report-emacs-build-time)))
             ("system" . ,system-type)
             ("window system" . ,window-system)
-            ,(and (fboundp 'Meadow-version)
-                  (cons "Meadow" (howm-funcall-if-defined (Meadow-version))))
+            ,(let ((f 'Meadow-version))
+               ;; cheat to avoid warning while byte-compilation.
+               (and (fboundp f)
+                    (cons "Meadow" (funcall f))))
+            ("ENV" . ,(mapconcat (lambda (v) (format "%s=%s" v (getenv v)))
+                                 '("LC_ALL" "LC_CTYPE" "LANGUAGE" "LANG")
+                                 ", "))
             )))
 
+(defun honest-report-emacs-build-time ()
+  (if (stringp emacs-build-time)
+      emacs-build-time  ;; xemacs
+    (format-time-string "%Y-%m-%d"
+                        emacs-build-time)))
+
 ;;;;;;;;;;;;;
 
 (provide 'honest-report)
