@@ -1,7 +1,7 @@
 ;;; howm-cl.el --- Wiki-like note-taking tool
-;;; Copyright (c) 2002, 2003, 2004, 2005, 2006
-;;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;;; $Id: howm-cl.el,v 1.4 2006/01/16 14:59:51 hira Exp $
+;;; Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012
+;;;   HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
+;;; $Id: howm-cl.el,v 1.14 2011-12-31 15:07:29 hira Exp $
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -35,7 +35,9 @@
 (defalias 'howm-cl-assoc-if          'assoc-if)
 (defalias 'howm-cl-find-if           'find-if)
 (defalias 'howm-cl-find-if-not       'find-if-not)
+(defalias 'howm-cl-gensym            'gensym)
 (defalias 'howm-cl-mapcan            'mapcan)
+(defalias 'howm-cl-mapcar*           'mapcar*)
 (defalias 'howm-cl-member*           'member*)
 (defalias 'howm-cl-member-if         'member-if)
 (defalias 'howm-cl-position          'position)
@@ -45,6 +47,10 @@
 (defalias 'howm-cl-remove-if-not     'remove-if-not)
 (defalias 'howm-cl-subseq            'subseq)
 
+(defun howm-cl-remove-duplicates* (&rest args)
+  ":from-end version of `remove-duplicates'"
+  (apply #'howm-cl-remove-duplicates `(,@args :from-end t)))
+
 (provide 'howm-cl)
 
 ;;; howm-cl.el ends here
