@@ -1,7 +1,7 @@
 ;;; howm-mode.el --- Wiki-like note-taking tool
-;;; Copyright (c) 2002, 2003, 2004, 2005, 2006
-;;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;;; $Id: howm-mode.el,v 1.290 2006/03/22 13:29:25 hira Exp $
+;;; Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012
+;;;   HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
+;;; $Id: howm-mode.el,v 1.317 2012-09-23 10:44:24 hira Exp $
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -35,7 +35,8 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Require
 
-(require 'howm-common)
+(provide 'howm-mode)
+(require 'howm)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Customize
@@ -44,7 +45,7 @@
 
 ;; You can easily modify them.
 
-(defvar howm-template
+(howm-defvar-risky howm-template
   (concat howm-view-title-header " %title%cursor\n%date %file\n\n")
   "Contents of new file. %xxx are replaced with specified items.
 If it is a list, <n>-th one is used when you type C-u <n> M-x howm-create.
@@ -73,7 +74,11 @@ If it is a function, it is called to get template string with the argument <n>."
     ("m" howm-list-migemo t t)
     ("t" howm-list-todo t t)
     ("y" howm-list-schedule t t)
+    ("b" howm-list-buffers t t)
+    ("x" howm-list-mark-ring t t)
+    ("o" howm-occur t t)
     ("c" howm-create t t)
+    ("e" howm-remember t t)
     ("," howm-menu t t)
     ("." howm-find-today nil t)
     (":" howm-find-yesterday nil t)
@@ -97,59 +102,51 @@ If it is a function, it is called to get template string with the argument <n>."
     ("w" howm-random-walk nil t)
     ("M" howm-open-named-file t t)
     )
-  "List of (key function list-mode-p).
+  "List of (key function list-mode-p global-p).
 `howm-prefix' + this key is real stroke.
 If optional argument list-mode-p is non-nil,
-same key is also available in view mode."
+same key is also available in view mode.
+It is further registered globally if global-p is non-nil."
   )
 
-(defvar howm-migemo-client nil)
-;; (defvar howm-migemo-client "migemo-client")
-(defvar howm-migemo-client-option nil
+(howm-defvar-risky howm-migemo-client nil
+  "Command name of migemo-client.
+Try (setq howm-migemo-client \"migemo-client\") for howm-migemo-*.")
+(howm-defvar-risky howm-migemo-client-option nil
   "List of option for migemo-client.
 e.g. (\"-H\" \"::1\")")
 
-;; (defvar howm-migemo-client nil
-;;   "Command name (string) of migemo-client. Nil means default command.
-;; Example:
-;;   (setq howm-migemo-client \"my-migemo-client-command\")
-;;   (setq howm-migemo-client '(\"migemo-client\" \"-H\" \"::1\")) ;; options
-;;   (setq howm-migemo-client '(nil \"-H\" \"::1\")) ;; same as the above")
-
 ;;; --- level 2 ---
 
 ;; Be careful to keep consistency.
 
-(defvar howm-keyword/ref-regexp-format "\\(%s\\)[ \t]*\\([^ \t\r\n].*\\)")
-(defvar howm-keyword-format (format "%s %%s" howm-keyword-header)
+(howm-defvar-risky howm-keyword/ref-regexp-format
+  "\\(%s\\)[ \t]*\\([^ \t\r\n].*\\)")
+(howm-defvar-risky howm-keyword-format
+  (format "%s %%s" howm-keyword-header)
   "Format for declaration of keyword. See `format'.")
-(defvar howm-keyword-regexp (format howm-keyword/ref-regexp-format
-                                   (regexp-quote howm-keyword-header)))
-(defvar howm-keyword-regexp-hilit-pos 1)
-(defvar howm-keyword-regexp-pos 2)
-(defvar howm-ref-regexp (format howm-keyword/ref-regexp-format
-                               (regexp-quote howm-ref-header))
+(howm-defvar-risky howm-keyword-regexp
+  (format howm-keyword/ref-regexp-format (regexp-quote howm-keyword-header)))
+(howm-defvar-risky howm-keyword-regexp-hilit-pos 1)
+(howm-defvar-risky howm-keyword-regexp-pos 2)
+(howm-defvar-risky howm-ref-regexp
+  (format howm-keyword/ref-regexp-format (regexp-quote howm-ref-header))
   "Regexp for explicit link.")
-(defvar howm-ref-regexp-hilit-pos 0
+(howm-defvar-risky howm-ref-regexp-hilit-pos 0
   "Position of search string in `howm-ref-regexp'")
-(defvar howm-ref-regexp-pos 2
+(howm-defvar-risky howm-ref-regexp-pos 2
   "Position of search string in `howm-ref-regexp'")
-(defvar howm-wiki-regexp "\\[\\[\\([^]\r\n]+\\)\\]\\]"
+(howm-defvar-risky howm-wiki-regexp "\\[\\[\\([^]\r\n]+\\)\\]\\]"
   "Regexp for explicit link.")
-(defvar howm-wiki-regexp-hilit-pos 1
+(howm-defvar-risky howm-wiki-regexp-hilit-pos 1
   "Position of hilight in `howm-wiki-regexp'")
-(defvar howm-wiki-regexp-pos 1
+(howm-defvar-risky howm-wiki-regexp-pos 1
   "Position of search string in `howm-wiki-regexp'")
-(defvar howm-wiki-format "[[%s]]"
+(howm-defvar-risky howm-wiki-format "[[%s]]"
   "Format for declaration of wiki word. See `format'.")
-;; (defvar howm-wiki-regexp "\\[\\[\\([^]\r\n]+\\)\\(\\]\\]\\)"
-;;   "Regexp for explicit link.")
-;; (defvar howm-wiki-regexp-hilit-pos 2
-;;   "Position of search string in `howm-wiki-regexp'")
-;; (defvar howm-wiki-regexp-pos 1
-;;   "Position of search string in `howm-wiki-regexp'")
 
 ;; Fix me: redundant (howm-date-* & howm-reminder-*)
+;; (cf.) howm-reminder-regexp-grep-* howm-reminder-today-format
 (defvar howm-date-regexp-grep
   (concat "[1-2][0-9][0-9][0-9]" howm-date-separator
           "[0-1][0-9]" howm-date-separator
@@ -170,7 +167,7 @@ e.g. (\"-H\" \"::1\")")
 (defvar howm-insert-date-format "[%s]")
 (defvar howm-insert-date-future nil)
 
-(defvar howm-template-rules
+(howm-defvar-risky howm-template-rules
   '(("%title" . howm-template-title)
     ("%date" . howm-template-date)
     ("%file" . howm-template-previous-file)
@@ -209,6 +206,15 @@ in `howm-template'. %s is replaced with name of last file. See `format'.")
 (defun howm-action-lock-date-rule ()
   (action-lock-general 'howm-action-lock-date howm-date-regexp 0 0))
 
+(defun howm-action-lock-quote-keyword (keyword)
+  (let ((q (regexp-quote keyword)))
+    ;; when a regexp is specified, leave unmatched keywords.
+    (if (and (stringp howm-check-word-break)
+             (not (string-match howm-check-word-break keyword)))
+        q
+      ;; add word break checks
+      (concat "\\b" q "\\b"))))
+
 (defun howm-action-lock-setup ()
   (setq action-lock-case-fold-search howm-keyword-case-fold-search)
   (action-lock-mode t)
@@ -216,17 +222,23 @@ in `howm-template'. %s is replaced with name of last file. See `format'.")
     ;; override the rule in action-lock.el
     (action-lock-add-rules (list date-al) t))
   (let* ((ks (howm-keyword-for-goto))
-         (r (mapconcat 'regexp-quote ks "\\|"))
+         (r (mapconcat (if howm-check-word-break
+                           #'howm-action-lock-quote-keyword
+                         #'regexp-quote)
+                       ks "\\|"))
+         ;; The following optimization causes an error
+         ;; "Variable binding depth exceeds max-specpdl-size".
+         ;; (r (cond ((stringp howm-check-word-break)
+         ;;           (mapconcat #'howm-action-lock-quote-keyword ks "\\|"))
+         ;;          (t
+         ;;           (regexp-opt ks (and howm-check-word-break 'word)))))
          (wiki (howm-action-lock-search howm-wiki-regexp
                                         howm-wiki-regexp-pos
                                         howm-wiki-regexp-hilit-pos
-                                        t
-                                        ))
-;;          (explicit (howm-action-lock-goto howm-ref-regexp
+                                        t))
          (explicit (howm-action-lock-search howm-ref-regexp
                                             howm-ref-regexp-pos
-                                            howm-ref-regexp-hilit-pos
-                                            ))
+                                            howm-ref-regexp-hilit-pos))
          (implicit (howm-action-lock-search r 0))
          (rev (howm-action-lock-related howm-keyword-regexp
                                         howm-keyword-regexp-pos
@@ -285,13 +297,12 @@ key	binding
 "
   nil ;; default = off
   howm-lighter ;; mode-line
-  `(
-    ,@(mapcar (lambda (entry)
-                (let ((k (car entry))
-                      (f (second entry)))
-                  (cons (concat howm-prefix k) f)))
-              howm-default-key-table)
-    ))
+  (mapcar (lambda (entry)
+            (let ((k (car entry))
+                  (f (second entry)))
+              (cons (concat howm-prefix k) f)))
+          howm-default-key-table)
+  )
 
 ;; emacs20's easy-mmode-define-minor-mode can't have body. sigh...
 (add-hook 'howm-mode-on-hook 'howm-initialize-buffer)
@@ -314,8 +325,7 @@ key	binding
             (when global-p
               (define-key global-map pk f))))
         howm-default-key-table)
-  (define-key howm-mode-map "\C-x\C-s" 'howm-save-buffer)
-  )
+  (define-key howm-mode-map "\C-x\C-s" 'howm-save-buffer))
 (howm-set-keymap)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -343,7 +353,7 @@ key	binding
       ;; I don't understand this. [2004-12-18]
       (howm-fontify t)
       ;; make-local-hook is obsolete for emacs >= 21.1.
-      (make-local-hook 'after-save-hook)
+      (howm-funcall-if-defined (make-local-hook 'after-save-hook))
       (add-hook 'after-save-hook 'howm-after-save t t))))
 
 (defun howm-after-save ()
@@ -362,24 +372,26 @@ key	binding
 (defun howm-list-all ()
   (interactive)
   (howm-set-command 'howm-list-all)
-  (howm-view-directory (car (howm-search-path)) t)
-  (howm-list-normalize)
-  ;; clean me [2004-07-11]
-  (cond ((howm-list-title-p) t)  ;; already done in (howm-list-normalize)
-        (howm-list-all-title (howm-list-title))))
+  (howm-normalize-show "" (howm-all-items))
+  ;; for backward compatibility
+  (cond ((howm-list-title-p) t)  ;; already done in howm-normalize-show
+        (howm-list-all-title (howm-list-title-internal))))
+
+(defun howm-all-items ()
+  "Returns list of all items in the first search path."
+  (howm-folder-items (car (howm-search-path)) t))
 
 (defun howm-list-recent (&optional days)
   (interactive "P")
   (howm-set-command 'howm-list-recent)
   (let* ((d (or days howm-list-recent-days))
          (now (current-time))
-         (from (howm-days-before now d)))
-    (howm-view-directory howm-directory t)
-    (howm-view-filter-by-mtime nil (list from now))
-    (howm-list-normalize)
+         (from (howm-days-before now d))
+         (item-list (howm-folder-items howm-directory t)))
+    (howm-normalize-show "" (howm-filter-items-by-mtime item-list from now))
     ;; clean me [2003-11-30]
-    (cond ((howm-list-title-p) t)  ;; already done in (howm-list-normalize)
-          (howm-list-recent-title (howm-list-title))
+    (cond ((howm-list-title-p) t)  ;; already done in howm-normalize-show
+          (howm-list-recent-title (howm-list-title-internal))
           ((not days) (howm-view-summary-to-contents)))))
 
 ;; clean me: direct access to howm-view-* is undesirable.
@@ -387,9 +399,11 @@ key	binding
 (defvar howm-list-title-previous nil
   "For internal use")
 (make-variable-buffer-local 'howm-list-title-previous)
-(defun howm-list-title-put-previous ()
+(defun howm-list-title-put-previous (&optional item-list)
   (when howm-list-title-undo
-    (setq howm-list-title-previous (howm-view-item-list))))
+    (setq howm-list-title-previous (or item-list (howm-view-item-list)))))
+(defun howm-list-title-clear-previous ()
+  (setq howm-list-title-previous nil))
 (defun howm-list-title-get-previous ()
   (if howm-list-title-undo
       (let ((prev howm-list-title-previous))
@@ -398,20 +412,23 @@ key	binding
     (error "Undo is not enabled.")))
 (defun howm-list-title-regexp ()
   (or howm-list-title-regexp (howm-view-title-regexp-grep)))
-(defun howm-list-title (&optional undo)
+(defalias 'howm-list-title 'howm-list-toggle-title) ;; backward compatibility
+(defun howm-list-toggle-title (&optional undo)
   (interactive "P")
   (if (or undo howm-list-title-previous)
       (howm-list-title-get-previous)
-    (let ((b (current-buffer)))
-      (howm-list-title-put-previous)
-      (howm-view-list-title (howm-list-title-regexp))
-;;       (howm-view-filter-by-contents (howm-list-title-regexp))
-      (let ((c (current-buffer)))
-        (when (not (eq b c))
-          (set-buffer b)
-          (howm-view-kill-buffer)
-          (switch-to-buffer c)
-          (howm-view-summary-check t))))))
+    (howm-list-title-internal)))
+(defun howm-list-title-internal ()
+  (let ((b (current-buffer)))
+    (howm-list-title-put-previous)
+    (howm-view-list-title (howm-list-title-regexp))
+    ;;       (howm-view-filter-by-contents (howm-list-title-regexp))
+    (let ((c (current-buffer)))
+      (when (not (eq b c))
+        (set-buffer b)
+        (howm-view-kill-buffer)
+        (switch-to-buffer c)
+        (howm-view-summary-check t)))))
 
 (defun howm-list-title-p ()
   (let ((a (howm-get-value howm-list-title)))
@@ -420,7 +437,7 @@ key	binding
           (t a))))
 
 (defun howm-days-after (ti days &optional hours)
-  (let* ((ne (decode-time ti))
+  (let* ((ne (howm-decode-time ti))
          (hour-pos 2)
          (day-pos 3)
          (nh (nth hour-pos ne))
@@ -450,13 +467,18 @@ key	binding
       (howm-write-history regexp))
     (howm-search regexp completion-p)))
 
-(defun howm-search (regexp fixed-p &optional emacs-regexp)
+(defun howm-search (regexp fixed-p &optional emacs-regexp filter)
   (if (string= regexp "")
       (howm-list-all)
     (howm-message-time "search"
-      (prog1
-          (howm-call-view-search regexp fixed-p emacs-regexp)
-        (howm-list-normalize (or emacs-regexp regexp))))))
+      (let* ((trio (howm-call-view-search-internal regexp fixed-p emacs-regexp))
+             (kw (car trio))
+             (name (second trio))
+             (items (third trio)))
+        (when filter
+          (setq items (funcall filter items)))
+        (howm-normalize-show name items (or emacs-regexp regexp) nil nil kw)
+        (howm-record-view-window-configuration)))))
 
 (defvar *howm-view-window-configuration* nil
   "For internal use")
@@ -472,13 +494,12 @@ key	binding
   (interactive)
   (howm-restore-view-window-configuration))
 
-(defun howm-call-view-search (regexp fixed-p &optional emacs-regexp)
+(defun howm-call-view-search-internal (regexp fixed-p &optional emacs-regexp)
   (let ((hilit (if emacs-regexp
                    `((,emacs-regexp . howm-view-hilit-face))
                  nil)))
-    (howm-view-search-folder regexp (howm-search-path-folder)
-                             nil nil fixed-p hilit)
-    (howm-record-view-window-configuration)))
+    (howm-view-search-folder-internal regexp (howm-search-path-folder)
+                                      nil nil fixed-p hilit)))
 
 (defun howm-list-migemo (&optional completion-p)
   (interactive "P")
@@ -522,22 +543,146 @@ key	binding
 ;; (defun howm-migemo-client-option ()
 ;;   (cdr-safe howm-migemo-client))
 
-(defun howm-list-normalize (&optional keyword comefrom-regexp no-list-title)
+(defun howm-normalize-oldp ()
+  howm-list-normalizer)
+
+;; ;; generate conv in howm-normalizer-pair
+;; (let ((methods '("random" "name" "numerical-name" "date" "reverse-date"
+;;                  "summary" "reminder" "mtime" "reverse")))
+;;   (mapcar (lambda (m)
+;;             (let ((command
+;;                    (howm-get-symbol nil "howm-view-sort-by-" m))
+;;                   (internal
+;;                    (howm-get-symbol nil "howm-sort-items-by-" m)))
+;;               (cons command internal)))
+;;           methods))
+
+(defun howm-normalizer-pair ()
+  (let* ((old howm-list-normalizer)
+         (new howm-normalizer)
+         (conv '((howm-view-sort-by-random . howm-sort-items-by-random)
+                 (howm-view-sort-by-name . howm-sort-items-by-name)
+                 (howm-view-sort-by-numerical-name
+                  . howm-sort-items-by-numerical-name)
+                 (howm-view-sort-by-date . howm-sort-items-by-date)
+                 (howm-view-sort-by-reverse-date
+                  . howm-sort-items-by-reverse-date)
+                 (howm-view-sort-by-summary . howm-sort-items-by-summary)
+                 (howm-view-sort-by-reminder . howm-sort-items-by-reminder)
+                 (howm-view-sort-by-mtime . howm-sort-items-by-mtime)
+                 (howm-view-sort-by-reverse . howm-sort-items-by-reverse)))
+         (p (assoc old conv))
+         (q (assoc new conv)))
+    (when q
+      (message "Warning: %s is wrong for howm-normalizer. Use %s." (car q) (cdr q))
+      (setq new (cdr q)))
+    (cond ((null old) (cons old new))
+          (p (cons nil (cdr p)))
+          (t (cons old #'identity)))))
+
+(put 'howm-with-normalizer 'lisp-indent-hook 0)
+(defmacro howm-with-normalizer (&rest body)
+  (let ((g (howm-cl-gensym)))
+    `(progn
+       (when (howm-normalize-oldp)
+         (message
+          "Warning: howm-list-normalizer is obsolete. Use howm-normalizer."))
+       (let* ((,g (howm-normalizer-pair))
+              (howm-list-normalizer (car ,g))
+              (howm-normalizer (cdr ,g)))
+         ,@body))))
+
+(defun howm-normalize-show (name item-list
+                                 &optional keyword comefrom-regexp no-list-title
+                                 fl-keywords)
+  ;; comefrom-regexp and no-list-title are never used now. [2009-07-23]
+  (howm-with-normalizer
+    (if (howm-normalize-oldp)
+        ;; for backward compatibility.
+        (progn
+          (howm-view-summary name item-list fl-keywords)
+          (howm-list-normalize-old keyword comefrom-regexp no-list-title))
+      (let* ((r (howm-normalize item-list keyword
+                                comefrom-regexp no-list-title)))
+        (howm-call-view-summary name (cdr r) fl-keywords)
+        (car r)))))
+
+(defun howm-call-view-summary (name item-list-pair fl-keywords)
+  (let ((orig (car item-list-pair))
+        (entitled (cdr item-list-pair)))
+    (howm-view-summary name (or entitled orig) fl-keywords)
+    ;; side effect
+    (if entitled
+        (howm-list-title-put-previous orig)
+      (howm-list-title-clear-previous))))
+
+(defun howm-normalize (item-list
+                       &optional keyword comefrom-regexp no-list-title)
+  ;; no-list-title is never used now. [2009-07-23]
+  "Sort ITEM-LIST in the standard order."
+  (let ((matched nil)
+        (entitled-item-list nil))
+    (setq item-list (funcall howm-normalizer item-list))
+    (when keyword
+      (let ((key-reg (or comefrom-regexp
+                         (howm-make-keyword-regexp1 keyword)))
+            (word-reg (format "\\<%s\\>"
+                              (if (stringp keyword)
+                                  (regexp-quote keyword)
+                                (regexp-opt keyword t))))
+            (wiki-reg (regexp-quote (howm-make-wiki-string keyword)))
+            (file-reg (and
+                       (stringp keyword)
+                       (format "^%s$"
+                               (regexp-quote (expand-file-name keyword)))))
+            (case-fold-search howm-keyword-case-fold-search))
+        (labels ((check (tag flag reg &optional tag-when-multi-hits)
+                        (when flag
+                          (let ((r (howm-normalize-check item-list tag reg
+                                                         tag-when-multi-hits)))
+                            (setq matched (append (car r) matched))
+                            (setq item-list (cdr r))))))
+          ;; not efficient. should I do them at once?
+          (check 'word            howm-list-prefer-word word-reg)
+          (check 'wiki            howm-list-prefer-wiki wiki-reg)
+          (check 'related-keyword t howm-keyword-regexp)
+          (check 'keyword         t key-reg 'keyword-multi-hits)
+          (check 'file            file-reg file-reg))))
+    (when (and (howm-list-title-p)
+               (not no-list-title)
+               (not (and (member 'file matched)
+                         howm-inhibit-title-file-match)))
+      (setq entitled-item-list
+            (howm-entitle-items (howm-list-title-regexp) item-list)))
+    (cons matched (cons item-list entitled-item-list))))
+
+(defun howm-normalize-check (item-list tag reg tag-when-multi-hits)
+  (let* ((r (if (eq tag 'file)
+                (howm-view-lift-by-path-internal item-list reg)
+              (howm-view-lift-by-summary-internal item-list reg)))
+         (m (car r))
+         (item-list (cdr r))
+         (matched (cond ((and tag-when-multi-hits (eq m 'multi))
+                         (list tag-when-multi-hits tag))
+                        (m (list tag))
+                        (t nil))))
+    (cons matched item-list)))
+
+(defun howm-list-normalize-old (&optional keyword comefrom-regexp no-list-title)
   "Sort displayed items in the standard order.
+This function is obsolete. Use `howm-normalize' insteadly.
 --- Sorry, below documentation is incomplete. ---
 When KEYWORD is given, matched items are placed on the top.
 KEYWORD can be a string or a list of strings.
 "
-;;     (let ((*howm-view-summary-nop* t)) ;; dirty
-  (let ((matched (howm-view-in-background
-                   (howm-list-normalize-subr keyword comefrom-regexp
-                                             no-list-title))))
-    (howm-view-summary)
-    matched))
+  (prog1
+      (howm-view-in-background
+        (howm-list-normalize-subr keyword comefrom-regexp no-list-title))
+    (howm-view-summary)))
 
 (defun howm-list-normalize-subr (keyword comefrom-regexp no-list-title)
+  "Obsolete. Do not use this any more."
   (let ((matched nil))
-    (howm-list-exclude)
     (funcall howm-list-normalizer)
     (when keyword
       (let ((key-reg (or comefrom-regexp
@@ -552,22 +697,27 @@ KEYWORD can be a string or a list of strings.
                        (format "^%s$"
                                (regexp-quote (expand-file-name keyword)))))
             (case-fold-search howm-keyword-case-fold-search))
-        (let ((check (lambda (tag flag reg)
-                       (and flag
-                            (if (eq tag 'file)
-                                (howm-view-sort-by-name-match nil reg t)
-                              (howm-view-sort-by-summary-match nil reg))
-                            (setq matched (cons tag matched))))))
+        ;; clean me.
+        (let ((check (lambda (tag flag reg &optional tag-when-multi-hits)
+                       (when flag
+                         (let ((m (if (eq tag 'file)
+                                      (howm-view-lift-by-name nil reg t)
+                                    (howm-view-lift-by-summary nil reg))))
+                           (when m
+                             (setq matched (cons tag matched)))
+                           (when (and tag-when-multi-hits (eq m 'multi))
+                             (setq matched
+                                   (cons tag-when-multi-hits matched))))))))
           (funcall check 'word            howm-list-prefer-word word-reg)
           (funcall check 'wiki            howm-list-prefer-wiki wiki-reg)
           (funcall check 'related-keyword t howm-keyword-regexp)
-          (funcall check 'keyword         t key-reg)
+          (funcall check 'keyword         t key-reg 'keyword-multi-hits)
           (funcall check 'file            file-reg file-reg))))
     (when (and (howm-list-title-p)
                (not no-list-title)
                (not (and (member 'file matched)
                          howm-inhibit-title-file-match)))
-      (howm-list-title))
+      (howm-list-title-internal))
     matched))
 
 (defun howm-make-keyword-string (keyword)
@@ -608,13 +758,16 @@ We need entire-match in order to
 
 (defun howm-list-related (str)
   (howm-set-command 'howm-list-related)
-  (let ((keys (mapcar (lambda (k)
-                        (if howm-keyword-case-fold-search
-                            (downcase k)
-                          k))
-                      (howm-subkeyword str))))
-    (howm-search (howm-make-keyword-string ".*") nil)
-    (howm-view-filter-by-summary nil (regexp-opt keys))))
+  (let* ((keys (mapcar (lambda (k)
+                         (if howm-keyword-case-fold-search
+                             (downcase k)
+                           k))
+                       (howm-subkeyword str)))
+         (filter `(lambda (items)
+                    (howm-filter-items-by-summary items ,(regexp-opt keys)))))
+    ;; Note that regexp-opt returns a regexp for emacs (not for grep).
+    (howm-search (howm-make-keyword-string ".*") nil nil filter)))
+
 (defun howm-subkeyword (str)
   (with-temp-buffer
     (insert str)
@@ -623,13 +776,13 @@ We need entire-match in order to
 (defun howm-list-around ()
   (interactive)
   (howm-set-command 'howm-list-around)
-  (let ((f (buffer-file-name)))
-    (howm-list-all)
-    (howm-view-sort-by-reverse-date)
+  (let ((f (buffer-file-name))
+        (item-list (howm-view-sort-by-reverse-date-internal
+                    (howm-all-items))))
+    (howm-view-summary "" item-list)
     (let ((pos (howm-cl-position-if (lambda (item)
-                                      (string= (howm-view-item-filename item)
-                                               f))
-                                    (howm-view-item-list))))
+                                      (string= (howm-item-name item) f))
+                                    item-list)))
       (goto-char (point-min))
       (when pos
         (forward-line pos)))
@@ -756,12 +909,12 @@ is necessary.")
   ;; which-template should be 1, 2, 3, ...
   (setq which-template (or which-template 1))
   (cond ((stringp howm-template) howm-template)
-        ((listp howm-template) (nth (- which-template 1) howm-template))
         ((functionp howm-template) (let ((args (if howm-template-receive-buffer
                                                    (list which-template
                                                          previous-buffer)
                                                  (list which-template))))
-                                     (apply howm-template args)))))
+                                     (apply howm-template args)))
+        ((listp howm-template) (nth (- which-template 1) howm-template))))
 
 (defun howm-replace (rules arg &optional beg end)
   (mapc (lambda (pair)
@@ -882,26 +1035,29 @@ is necessary.")
       (let ((ans nil))
         (goto-char (point-min))
         (while (search-forward howm-keyword-list-alias-sep nil t)
-          (let ((line (buffer-substring-no-properties (line-beginning-position)
-                                                      (line-end-position))))
-            (setq ans
-                  (cons (split-string line howm-keyword-list-alias-sep) ans))
+          (let* ((line (buffer-substring-no-properties (line-beginning-position)
+                                                       (line-end-position)))
+                 (keys (split-string line howm-keyword-list-alias-sep))
+                 (ks (if howm-keyword-case-fold-search
+                         (mapcar #'downcase keys)
+                       keys)))
+            (setq ans (cons ks ans))
             (end-of-line)))
         ans))))
 
 (defun howm-expand-aliases-recursively (keyword aliases)
   (let ((keys (list keyword))
         (prev nil))
-    (flet ((expand (keys)
-                   (sort (howm-cl-remove-duplicates
-                          (howm-cl-mapcan (lambda (k)
-                                            (howm-cl-mapcan
-                                             (lambda (a) (if (member k a)
-                                                             (copy-list a)
-                                                           nil))
-                                             aliases))
-                                          keys) :test #'string=)
-                         #'string<)))
+    (labels ((expand (keys)
+                     (sort (howm-cl-remove-duplicates
+                            (howm-cl-mapcan (lambda (k)
+                                              (howm-cl-mapcan
+                                               (lambda (a) (if (member k a)
+                                                               (copy-sequence a)
+                                                             nil))
+                                               aliases))
+                                            keys) :test #'string=)
+                           #'string<)))
       (while (not (equal prev keys))
         (setq prev keys)
         (setq keys (expand keys))))
@@ -917,62 +1073,83 @@ is necessary.")
   "List of strings which are equivalent to KEYWORD.
 KEYWORD itself is always at the head of the returneded list.
 "
-  (let* ((aliases (howm-aliases))
+  ;; Return the original keyword (not downcased) for backward compatibility.
+  ;; I'm not sure whether this behavior is really needed.
+  (let* ((key (if howm-keyword-case-fold-search
+                  (downcase keyword)
+                keyword))
+         (aliases (howm-aliases))
          (equiv (if howm-keyword-aliases-recursive
-                    (howm-expand-aliases-recursively keyword aliases)
+                    (howm-expand-aliases-recursively key aliases)
                   (howm-cl-remove-duplicates
                    (apply #'append
-                          (howm-cl-remove-if-not (lambda (a) (member keyword a))
+                          (howm-cl-remove-if-not (lambda (a) (member key a))
                                                  aliases))))))
     (if (null equiv)
         keyword
-      (cons keyword (remove keyword equiv)))))
+      (cons keyword (remove key equiv)))))
 
 (defun howm-keyword-search (keyword &optional create-p open-unique-p)
   (howm-message-time "key-search"
     (howm-set-command 'howm-keyword-search)
-    (howm-keyword-search-subr keyword create-p open-unique-p)))
+    (howm-with-normalizer
+      (howm-keyword-search-subr keyword create-p open-unique-p))))
 
 (defun howm-keyword-search-subr (keyword create-p open-unique-p)
-  (let* ((wconf (current-window-configuration))
-         (aliases (if (howm-support-aliases-p)
+  (let* ((aliases (if (howm-support-aliases-p)
                       (howm-keyword-aliases keyword)
                     keyword))
          (menu-p (howm-menu-keyword-p keyword))
-         (comefrom-regexp (if menu-p
+         (comefrom-regexp (if menu-p ;; clean me
                               nil
                             (howm-make-keyword-regexp2 aliases)))
-         (found (let ((howm-search-other-dir (if menu-p
+         (trio (let ((howm-search-other-dir (if menu-p ;; clean me
                                                  nil
                                                howm-search-other-dir))
-                      (*howm-view-force-case-fold-search* howm-keyword-case-fold-search)) ;; dirty!
-                  (howm-call-view-search aliases t)))
+                      (*howm-view-force-case-fold-search*
+                       howm-keyword-case-fold-search)) ;; dirty!
+                 (howm-call-view-search-internal aliases t)))
 ;; code for <http://pc8.2ch.net/test/read.cgi/unix/1077881095/823>.
 ;; but this change is canceled; I'll try more fundamental fix. [2005-11-04]
 ;;                   (if open-unique-p
 ;;                       (let ((r (concat "^" (regexp-quote keyword) "$")))
 ;;                         (howm-call-view-search r nil))
 ;;                     (howm-call-view-search aliases t))))
-         (matched (and found (howm-list-normalize aliases comefrom-regexp t)))
+         (kw (car trio))
+         (name (second trio))
+         (items (third trio))
+         (items-pair nil)
+         (found (if items t nil)) ;; want to forget items as soon as possible
+         (matched (and found
+                       (let* ((howm-keyword-format
+                               (if menu-p ;; clean me
+                                   (default-value 'howm-keyword-format)
+                                 howm-keyword-format))
+                              (r (howm-normalize items aliases
+                                                 comefrom-regexp)))
+                         (setq items-pair (cdr r))
+                         (car r))))
          (keyword-matched (member 'keyword matched))
+         (keyword-matched-multi (member 'keyword-multi-hits matched))
          (file-matched (member 'file matched))
          (title (howm-make-keyword-string keyword)))
     ;; main processing (clean me!) [2003-12-01]
     (cond
      ;; for %foo%
      ((and menu-p keyword-matched)
-      (howm-keyword-search-open-menu keyword wconf))
+      (howm-keyword-search-open-menu keyword (car items-pair)
+                                     keyword-matched-multi))
      ;; for [[foo]]
      ((and create-p (not keyword-matched))
       (howm-keyword-search-create title))
      ;; open if unique match
-     ((and open-unique-p (howm-single-element-p (howm-view-item-list)))
-      (howm-keyword-search-open-unique wconf))
-     ;; show title
-     ((and found
-           (howm-list-title-p)
-           (not (and file-matched howm-inhibit-title-file-match)))
-      (howm-list-title)))
+     ((and open-unique-p (howm-single-element-p items))
+      (howm-keyword-search-open-unique items))
+     (t
+      (howm-call-view-summary name items-pair kw)
+      (when (howm-normalize-oldp)
+        ;; sorry for redundancy & inefficiency
+        (howm-list-normalize-old aliases comefrom-regexp t))))
     ;; record history
     (when (not menu-p)
       (howm-write-history keyword))
@@ -984,42 +1161,39 @@ KEYWORD itself is always at the head of the returneded list.
       (create-p . ,create-p))
     ))
 
-(defun howm-keyword-search-open-menu (keyword wconf)
+(defun howm-keyword-search-open-menu (keyword item-list multi-hits-p)
   "Open KEYWORD as menu."
   ;; dirty. peeking howm-view.el
-  (let* ((item (car (howm-view-item-list)))
+  (let* ((item (car item-list))
          (fname (howm-view-item-filename item))
          (place (howm-view-item-place item)))
-    (howm-view-kill-buffer)
-    (set-window-configuration wconf)
-    (howm-menu-open fname place
-                    (howm-menu-name keyword))))
+    (let ((howm-search-other-dir nil))
+      (howm-menu-open fname place (howm-menu-name keyword))))
+  (when multi-hits-p
+    (message "Warning: found two or more %s." keyword)))
 
 (defun howm-keyword-search-create (title)
   "create new memo <<< TITLE."
-  (howm-view-kill-buffer)
   (howm-create-file-with-title title)
   (message "New keyword."))
 
-(defun howm-keyword-search-open-unique (wconf)
+(defun howm-keyword-search-open-unique (items)
   "Open unique match."
-  ;; dirty. peeking howm-view.el
-  (howm-view-summary-open)
-  (let ((b (current-buffer)))
-    (set-window-configuration wconf)
-    (switch-to-buffer b)))
+  (howm-view-open-item (car items)))
 
 ;; (defvar *howm-keyword-buffer* nil) ;; for internal use
-(defun howm-keyword-for-goto ()
+(defun howm-keyword-for-goto (&optional keyword-list)
   (save-excursion
-    (let ((keys (howm-keyword-list))
-          (case-fold-search howm-keyword-case-fold-search))
+    (let ((case-fold-search howm-keyword-case-fold-search))
       (sort (howm-cl-mapcan (lambda (k)
                               (goto-char (point-min))
+                              ;; when howm-check-word-break is non-nil,
+                              ;; checking word breaks is desired for efficiency.
+                              ;; it is not implemented yet.
                               (if (search-forward k nil 'noerr)
                                   (list k)
                                 nil))
-                            keys)
+                            (or keyword-list (howm-keyword-list)))
             (lambda (x y)
               (> (length x) (length y)))))))
 
@@ -1037,6 +1211,16 @@ KEYWORD itself is always at the head of the returneded list.
           (setq keyword-list (cons key-str keyword-list))))
       (howm-keyword-add keyword-list)
       (message "%s" m))))
+(defun howm-keyword-add-items (items)
+  (let ((files (mapcar #'howm-view-item-filename items)))
+    (with-temp-buffer
+      (setq default-directory dir)
+      (mapc (lambda (f)
+              (erase-buffer)
+              (insert-file-contents f)
+              (howm-set-configuration-for-file-name f)
+              (howm-keyword-add-current-buffer))
+            files))))
 
 (defun howm-keyword-read ()
   (let ((ks nil)
@@ -1049,27 +1233,4 @@ KEYWORD itself is always at the head of the returneded list.
     (end-of-line)
     ks))
 
-;; (defun howm-keyword-read ()
-;;   (prog1
-;;       (howm-keyword-read-string (buffer-substring-no-properties
-;;                                  (line-beginning-position)
-;;                                  (line-end-position)))
-;;     (end-of-line)))
-
-;; (defun howm-keyword-read-string (str)
-;;   (with-temp-buffer
-;;     (insert str)
-;;     (goto-char (point-max))
-;;     (let ((ans nil))
-;;       (while (re-search-backward howm-keyword-regexp nil t)
-;;         (let* ((mbeg (match-beginning 0))
-;;                (key (match-string-no-properties howm-keyword-regexp-pos)))
-;;           (setq ans (cons key ans))
-;;           (delete-region mbeg (point-max))
-;;           (skip-chars-backward " ")
-;;           (delete-region (point) (point-max))))
-;;       ans)))
-
-(provide 'howm-mode)
-
 ;;; howm-mode.el ends here
