@@ -1,7 +1,7 @@
 ;;; howm-reminder.el --- Wiki-like note-taking tool
-;;; Copyright (c) 2002, 2003, 2004, 2005, 2006
-;;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;;; $Id: howm-reminder.el,v 1.58 2006/01/23 14:40:00 hira Exp $
+;;; Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012
+;;;   HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
+;;; $Id: howm-reminder.el,v 1.82 2011-12-31 15:07:29 hira Exp $
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -19,14 +19,15 @@
 ;;; USA.
 ;;--------------------------------------------------------------------
 
-(require 'howm-common)
+(provide 'howm-reminder)
+(require 'howm)
 
 (defvar howm-list-schedule-name "{schedule}")
 (defvar howm-list-todo-name "{todo}")
 ;   "This is used for buffer name of `howm-list-reminder'.
 ; See `howm-view-summary-name'.")
 
-(defvar howm-todo-priority-func
+(howm-defvar-risky howm-todo-priority-func
       '(("-" . howm-todo-priority-normal)
         (" " . howm-todo-priority-normal)
         ("+" . howm-todo-priority-todo)
@@ -47,7 +48,10 @@
 (defvar howm-todo-priority-todo-bottom     (- howm-huge))
 (defvar howm-todo-priority-defer-bottom    (- howm-huge))
 (defvar howm-todo-priority-deadline-bottom (- howm-huge))
-(defvar howm-todo-priority-schedule-bottom (- howm-huge))
+(defvar howm-todo-priority-schedule-bottom (- howm-huge++)
+  "Base priority of schedules in the bottom.
+Its default value is extremely negative so that you never see
+schedules outside the range in %reminder in the menu.")
 (defvar howm-todo-priority-deadline-top    howm-huge)
 (defvar howm-todo-priority-schedule-top    howm-huge)
 (defvar howm-todo-priority-unknown-top     howm-huge+)
@@ -82,12 +86,14 @@
       (defvar howm-reminder-regexp-type-pos 5)
       (defvar howm-reminder-regexp-laziness-pos 6)
       (defvar howm-reminder-today-format "@[%Y/%m/%d]")
-      (defvar howm-reminder-font-lock-keywords
+      (howm-defvar-risky howm-reminder-font-lock-keywords
         `(
           (,(howm-reminder-regexp "[-]?") (0 howm-reminder-normal-face prepend))
           (,(howm-reminder-regexp "[+]") (0 howm-reminder-todo-face prepend))
           (,(howm-reminder-regexp "[~]") (0 howm-reminder-defer-face prepend))
-          (,(howm-reminder-regexp "[!]") (0 howm-reminder-deadline-face prepend))
+          (,(howm-reminder-regexp "[!]")
+           (0 howm-reminder-deadline-face prepend)
+           (,howm-reminder-regexp-type-pos (howm-reminder-deadline-type-face) prepend))
           (,(howm-reminder-regexp "[@]") (0 howm-reminder-schedule-face prepend))
           (,(howm-reminder-regexp "[.]") (0 howm-reminder-done-face prepend))
           ))
@@ -146,12 +152,14 @@
     (defvar howm-reminder-regexp-laziness-pos 7)
     (defvar howm-reminder-today-format
       (format howm-insert-date-format howm-date-format))
-    (defvar howm-reminder-font-lock-keywords
+    (howm-defvar-risky howm-reminder-font-lock-keywords
       `(
         (,(howm-reminder-regexp "[-]") (0 howm-reminder-normal-face prepend))
         (,(howm-reminder-regexp "[+]") (0 howm-reminder-todo-face prepend))
         (,(howm-reminder-regexp "[~]") (0 howm-reminder-defer-face prepend))
-        (,(howm-reminder-regexp "[!]") (0 howm-reminder-deadline-face prepend))
+        (,(howm-reminder-regexp "[!]")
+         (0 howm-reminder-deadline-face prepend)
+         (,howm-reminder-regexp-type-pos (howm-reminder-deadline-type-face) prepend))
         (,(howm-reminder-regexp "[@]") (0 howm-reminder-schedule-face prepend))
         (,(howm-reminder-regexp "[.]") (0 howm-reminder-done-face prepend))
         ))
@@ -211,53 +219,14 @@
                                   (format ". %s" howm-reminder-cancel-string)))
     (defun howm-action-lock-done-modify (date type lazy desc)
       (concat date type lazy desc))
-;;     (defun howm-action-lock-done (&optional command)
-;;       (let ((whole-beg (match-beginning 0))
-;;             (com-beg (match-beginning howm-reminder-regexp-command-pos))
-;;             (com-end (match-end  howm-reminder-regexp-command-pos))
-;;             (type-beg (match-beginning howm-reminder-regexp-type-pos))
-;;             (type-end (match-end howm-reminder-regexp-type-pos))
-;;             (lazy-beg (match-beginning howm-reminder-regexp-laziness-pos))
-;;             (lazy-end (match-end howm-reminder-regexp-laziness-pos)))
-;;           (let* ((s (or command
-;;                         (howm-read-string (howm-action-lock-done-prompt)
-;;                                           "x-+~!.@"
-;;                                           "0123456789")))
-;; ;;                         (read-from-minibuffer (howm-action-lock-done-prompt) )))
-;;                  (c (cond ((string= s "") (progn (howm-congrats) "."))
-;;                           ((string= s "x") (format ". %s"
-;;                                                    howm-reminder-cancel-string))
-;;                           (t nil)))
-;;                  (type-or-lazy (string-match (format "^\\(%s?\\)\\([0-9]*\\)$"
-;;                                                      howm-reminder-types)
-;;                                              s)))
-;;             (cond (c
-;;                    (progn
-;;                      (goto-char com-beg)
-;;                      (insert ":")
-;; ;;                      (goto-char com-end)
-;; ;;                      (insert ")")
-;; ;;                      (goto-char com-beg)
-;; ;;                      (insert "(")
-;;                      (goto-char whole-beg)
-;;                      (insert (howm-reminder-today))
-;;                      (insert (format "%s " c))))
-;;                   (type-or-lazy
-;;                    (let* ((ty (match-string-no-properties 1 s))
-;;                           (lz (match-string-no-properties 2 s))
-;;                           (orig-ty (buffer-substring-no-properties type-beg
-;;                                                                    type-end))
-;;                           (orig-lz (buffer-substring-no-properties lazy-beg
-;;                                                                    lazy-end))
-;;                           (new-ty (if (string= ty "") orig-ty ty))
-;;                           (new-lz (if (string= lz "") orig-lz lz)))
-;;                      (goto-char com-beg)
-;;                      (delete-region com-beg com-end)
-;;                      (insert new-ty new-lz)))
-;;                   (t (error "Can't understand %s" s)))
-;;             (goto-char com-beg))))
     ))
 
+(defun howm-reminder-deadline-type-face ()
+  (let ((late (cadr (howm-todo-parse-string (match-string-no-properties 0)))))
+    (if (>= late 0)
+        howm-reminder-late-deadline-face
+      howm-reminder-deadline-face)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Reminder: schedule & todo
 
@@ -274,43 +243,50 @@
 
 (defun howm-list-schedule ()
   (interactive)
-  (howm-with-schedule-summary-format
-    (howm-list-reminder howm-schedule-types howm-list-schedule-name)
-    (howm-schedule-sort-by-date)
-    (howm-reminder-goto-today)))
+  (howm-with-need
+    (howm-with-schedule-summary-format
+      (let ((items (need (howm-list-reminder-internal howm-schedule-types))))
+        (howm-list-reminder-final-setup howm-list-schedule-name
+                                        (howm-schedule-sort-items items)))
+      (howm-reminder-goto-today)
+      (howm-view-summary-check))))
 
-(defun howm-list-reminder (types name)
+(defun howm-list-reminder-internal (types)
   (let* ((r (howm-reminder-regexp types))
          (rg (howm-reminder-regexp-grep types))
          (summarizer (howm-reminder-summarizer r t))
          (folder (howm-reminder-search-path-folder)))
-    (howm-view-search-folder rg folder name summarizer)
-    (howm-list-exclude)
-    (howm-reminder-add-font-lock)
-    (howm-mode-add-font-lock)
-    (let ((action-lock-default-rules
-           (howm-action-lock-reminder-forward-rules t)))
-      (action-lock-mode))))
+    (third (howm-view-search-folder-internal rg folder nil summarizer))))
 
-(defcustom howm-highlight-date-regexp-format
-  (regexp-quote howm-reminder-today-format)
-  "Time format for highlight of today and tommorow.
+(defun howm-list-reminder-final-setup (&optional name item-list)
+  (howm-view-summary name item-list
+                     (append (howm-reminder-add-font-lock-internal)
+                             (howm-mode-add-font-lock-internal)))
+  (let ((action-lock-default-rules
+         (howm-action-lock-reminder-forward-rules t)))
+    (action-lock-mode t)))
+
+(let ((rs (mapcar #'regexp-quote
+                  (list howm-date-format howm-reminder-today-format))))
+  (defcustom howm-highlight-date-regexp-format (car rs)
+    "Time format for highlight of today and tommorow.
 This value is passed to `format-time-string', and the result must be a regexp."
-  :type (let ((cs (mapcar (lambda (f) `(const ,(regexp-quote f)))
-                          (list howm-reminder-today-format howm-date-format))))
-          `(radio ,@cs
-                  string))
-  :group 'howm-experimental)
+    :type `(radio ,@(mapcar (lambda (r) `(const ,r)) rs)
+                    string)
+    :group 'howm-faces))
 
-(defun howm-reminder-add-font-lock ()
+(defun howm-reminder-today-font-lock-keywords ()
   (let ((today    (howm-reminder-today 0 howm-highlight-date-regexp-format))
         (tomorrow (howm-reminder-today 1 howm-highlight-date-regexp-format)))
-    (cheat-font-lock-append-keywords (howm-reminder-font-lock-keywords))
-    (cheat-font-lock-append-keywords `((,today
-                                        (0 howm-reminder-today-face prepend))))
-    (cheat-font-lock-append-keywords `((,tomorrow
-                                        (0 howm-reminder-tomorrow-face prepend))))
-    ))
+    `((,today (0 howm-reminder-today-face prepend))
+      (,tomorrow (0 howm-reminder-tomorrow-face prepend)))))
+
+(defun howm-reminder-add-font-lock ()
+  (cheat-font-lock-append-keywords (howm-reminder-add-font-lock-internal)))
+
+(defun howm-reminder-add-font-lock-internal ()
+  (append (howm-reminder-font-lock-keywords)
+          (howm-reminder-today-font-lock-keywords)))
 
 (defun howm-reminder-omit-before (regexp str)
   (string-match regexp str)
@@ -322,10 +298,10 @@ This value is passed to `format-time-string', and the result must be a regexp."
 ;;                 (string-match ,regexp content)
 ;;                 (substring content (match-beginning 0)))))
        ,(if show-day-of-week
-            '(format "%s %s"
-                     (howm-day-of-week-string (nth 4
-                                                   (howm-todo-parse-string s)))
-                     s)
+            '(let* ((p (howm-todo-parse-string s))
+                    (late (floor (nth 1 p)))
+                    (dow (howm-day-of-week-string (nth 4 p))))
+               (format "%s%3s %s" dow late s))
           's))))
 
 (defun howm-reminder-today (&optional days-after fmt)
@@ -363,16 +339,15 @@ This value is passed to `format-time-string', and the result must be a regexp."
                                       raw)))
     (howm-schedule-sort-items filtered)))
 
-(defun howm-schedule-sort-items (items)
-  (interactive)
+(defun howm-schedule-sort-items (items &optional reverse-p)
+  (when reverse-p
+    (error "Not supported."))
   (howm-with-schedule-summary-format
     (howm-sort #'howm-schedule-sort-converter #'howm-schedule-sort-comparer
                items)))
 (defun howm-schedule-sort-by-date ()
   (interactive)
-  (howm-with-schedule-summary-format
-    (howm-view-sort-general #'howm-schedule-sort-converter
-                            #'howm-schedule-sort-comparer)))
+  (howm-view-sort-doit #'howm-schedule-sort-items))
 (defun howm-schedule-sort-converter (item)
   (let ((z (howm-reminder-parse item)))
     (cons (car z)
@@ -391,16 +366,38 @@ This value is passed to `format-time-string', and the result must be a regexp."
   (let* ((r (howm-reminder-regexp types))
          (rg (howm-reminder-regexp-grep types))
          (summarizer (howm-reminder-summarizer r))
-         (folder (howm-reminder-search-path-folder))
-         (raw (howm-view-search-folder-items rg folder summarizer)))
-;;          (raw (howm-view-search-items rg howm-directory summarizer)))
-    (howm-cl-remove-if #'howm-item-exclude-p raw)))
+         (folder (howm-reminder-search-path-folder)))
+    (howm-view-search-folder-items rg folder summarizer)))
 
 (defun howm-list-todo ()
   (interactive)
-  (howm-with-schedule-summary-format
-    (howm-list-reminder howm-todo-types howm-list-todo-name)
-    (howm-todo-sort-by-priority)))
+  (howm-list-todo-sub))
+
+;; experimental [2006-06-26]
+(defun howm-todo-sleeping-p (item)
+  ;; (- howm-huge-) should be replaced with an appropreate variable.
+  (< (howm-todo-priority item) (- howm-huge-)))
+(defun howm-list-active-todo ()
+  (interactive)
+  (howm-list-todo-sub (lambda (item)
+                        (not (howm-todo-sleeping-p item)))))
+(defun howm-list-sleeping-todo ()
+  (interactive)
+  (howm-list-todo-sub #'howm-todo-sleeping-p))
+
+(defun howm-list-todo-sub (&optional pred)
+  (howm-with-need
+    (howm-with-schedule-summary-format
+      (let ((items (need (howm-list-reminder-internal howm-todo-types))))
+        (when pred
+          (setq items
+                (need (howm-cl-remove-if-not pred items))))
+        (setq items (howm-todo-sort-items items))
+        (when howm-todo-separators
+          (setq items
+                (howm-todo-insert-separators items
+                                             howm-todo-separators)))
+      (howm-list-reminder-final-setup howm-list-todo-name items)))))
 
 (defun howm-todo-menu (n limit-priority separators)
   "Find top N todo items, or all todo items if N is nil.
@@ -413,34 +410,36 @@ See docstring of the variable `howm-menu-reminder-separators' for details."
                                       (< (howm-todo-priority item)
                                          limit-priority))
                                     (howm-reminder-search howm-todo-menu-types)))
-         (sorted (howm-sort #'howm-todo-priority-ext
-                            #'howm-todo-priority-ext-gt
-                            cutted)))
+         (sorted (howm-todo-sort-items cutted)))
     (howm-todo-insert-separators (if n (howm-first-n sorted n) sorted)
-                                 separators)))
+                                 separators t)))
 
 (defun howm-reminder-menu (n limit-priority separators)
   (howm-with-reminder-setting
-    (howm-todo-menu howm-menu-todo-num
-                    limit-priority
-                    separators)))
+    (howm-todo-menu n limit-priority separators)))
 
-(defun howm-todo-insert-separators (item-list separators)
+(defun howm-todo-insert-separators (item-list separators
+                                              &optional relative-date-p)
   (let ((is (mapcar (lambda (item) (cons (howm-todo-priority item) item))
                     item-list))
         (sep (mapcar (lambda (pair)
-                       (cons (- howm-todo-priority-schedule-top
-                                (or (car pair) howm-huge-))
-                             (cdr pair)))
+                       (cons (if relative-date-p
+                                 (- howm-todo-priority-schedule-top
+                                    (or (car pair) howm-huge-))
+                               (or (car pair) (- howm-huge-)))
+                             (howm-make-item (howm-make-page:nil) (cdr pair))))
                      separators)))
     (mapcar #'cdr
             (sort (append is sep) #'(lambda (x y) (> (car x) (car y)))))))
 
+(defun howm-todo-sort-items (items &optional reverse-p)
+  (when reverse-p
+    (error "Not supported."))
+  (howm-sort #'howm-todo-priority-ext #'howm-todo-priority-ext-gt
+             items))
+
 (defun howm-todo-sort-by-priority ()
-  (interactive)
-  (howm-with-schedule-summary-format
-    (howm-view-sort-general #'howm-todo-priority-ext
-                            #'howm-todo-priority-ext-gt)))
+  (howm-view-sort-doit #'howm-todo-sort-items))
 
 ;; Clean me.
 (defun howm-reminder-parse (item)
@@ -604,20 +603,18 @@ Return true if E1 has higher priority than E2."
 When D is omitted, the current time is encoded.
 When D is t, the beginning of today is encoded."
   (let* ((e (apply #'encode-time (cond ((eq d t)
-                                        (let ((now (decode-time)))
+                                        (let ((now (howm-decode-time)))
                                           (append '(0 0 0) (cdddr now))))
                                        (d
                                         (mapcar #'string-to-number
                                                 (list "0" "0" "0" d m y)))
                                        (t
-                                        (decode-time)))))
+                                        (howm-decode-time)))))
          (hi (car e))
          (low (second e))
          (daysec (* 60 60 24.0)))
     (+ (* hi (/ 65536 daysec)) (/ low daysec))))
 
-(defvar howm-congrats-command nil)
-;; (setq howm-congrats-command '("play" "~/sound/level.wav"))
 (defun howm-congrats ()
   (setq howm-congrats-count (1+ howm-congrats-count))
   (let* ((n (length howm-congrats-format))
@@ -653,7 +650,7 @@ When D is t, the beginning of today is encoded."
 ;;; direct manipulation of items from todo list
 
 ;; I'm sorry for dirty procedure here.
-;; If we use naive howm-date-regexp, it matches to file name "2004-05-11.howm"
+;; If we use naive howm-date-regexp, it matches to file name "2004-05-11.txt"
 ;; in summary mode.
 (defun howm-action-lock-reminder-forward-rules (&optional summary-mode-p)
   (let* ((action-maker (lambda (pos)
@@ -719,31 +716,59 @@ When D is t, the beginning of today is encoded."
           "$"))
 
 (defun howm-action-lock-forward-invoke (form-reg cursor-reg)
+  (howm-modify-in-background (lambda (&rest dummy)
+                               ;; open the target file
+                               ;; and go to the corresponding line
+                               (howm-action-lock-forward-open))
+                             (lambda (form-reg cursor-reg)
+                               (howm-action-lock-forward-modify-current-line
+                                form-reg cursor-reg))
+                             howm-action-lock-forward-save-buffer
+                             howm-action-lock-forward-kill-buffer
+                             form-reg
+                             cursor-reg))
+
+(defun howm-modify-in-background (opener modifier save-p kill-p &rest args)
   (save-excursion
     (save-window-excursion
       (let ((original-buffers (buffer-list)))
-        (howm-action-lock-forward-open)
-        ;; Now we are on the corresponding line in the target file.
+        (apply opener args)
+        ;; We are in the target buffer now.
         (let ((initially-modified-p (buffer-modified-p)))
           (prog1
-              ;; Let's modify this line.
-              (howm-action-lock-forward-modify-current-line form-reg cursor-reg)
-            ;; And save/kill buffer if required.
-            (when (and howm-action-lock-forward-save-buffer
+              (apply modifier args)
+            (when (and save-p
                        (not initially-modified-p)
                        (buffer-modified-p))
               (save-buffer))
-            (when (and howm-action-lock-forward-kill-buffer
+            (when (and kill-p
                        (not (buffer-modified-p))
                        (not (member (current-buffer) original-buffers)))
-              (kill-buffer (current-buffer))))))))
-  ;; We are back to the original line in the original buffer.
-  )
+              (kill-buffer (current-buffer)))))))))
 
 (defun howm-action-lock-forward-modify-current-line (form-reg cursor-reg)
-  (end-of-line)
-  (when (not (re-search-backward form-reg (line-beginning-position) t))
-    (error "Can't find corresponding line."))
+  (howm-modify-form #'action-lock-invoke form-reg cursor-reg))
+
+(defun howm-modify-form (proc form-reg cursor-reg &rest args)
+  (labels
+      ((f-cursor ()
+                 (beginning-of-line)
+                 (re-search-forward cursor-reg
+                                    (line-end-position
+                                     (+ 1 howm-action-lock-forward-fuzziness))
+                                    t))
+       (b-cursor ()
+                 (end-of-line)
+                 (re-search-backward cursor-reg
+                                     (line-beginning-position
+                                      (- 1 howm-action-lock-forward-fuzziness))
+                                     t))
+       (b-form ()
+               (end-of-line)
+               (re-search-backward form-reg (line-beginning-position) t)))
+    (or (save-excursion (and (f-cursor) (b-form)))
+        (save-excursion (and (b-cursor) (b-form)))
+        (error "Can't find corresponding line.")))
   (goto-char (match-beginning 0))
   ;; Now we are at the beginning of the form.
   ;; Remember this position to report the modified tail.
@@ -751,10 +776,18 @@ When D is t, the beginning of today is encoded."
     (when (not (re-search-forward cursor-reg (line-end-position) t))
       (error "Can't find corresponding string."))
     (goto-char (match-beginning 0))
+    ;; update display. I don't understand why this is needed.
+    ;; Without this, cursor is placed at the end of buffer if I delete many
+    ;; lines before the form position in the below setting (GNU Emacs 21.4.1).
+    ;;   (setq howm-menu-refresh-after-save nil)
+    ;;   (setq howm-menu-expiry-hours 3)
+    ;;   (setq howm-action-lock-forward-fuzziness 20000)
+    ;; Sigh...
+    (switch-to-buffer (current-buffer) t)
     ;; Now we are at the corresponding position.
-    ;; Let's hit magic-return!
+    ;; Let's call proc to modify the form!
     (undo-boundary)
-    (action-lock-invoke))
+    (apply proc args))
   ;; We are back to the beginning of the form.
   ;; Report the modified tail.
   (buffer-substring-no-properties (point) (line-end-position)))
@@ -778,6 +811,44 @@ When D is t, the beginning of today is encoded."
         (t
          (error "Not supported on this buffer."))))
 
+;;; extend deadlines (experimental)
+
+(put 'howm-extend-deadlines 'disabled t)
+(defun howm-extend-deadlines (days)
+  "Extend all overdue deadlines for DAYS from today."
+  (interactive "nHow many days? ")
+  (let ((hit (howm-cl-remove-if (lambda (item)
+                             (< (second (howm-reminder-parse item)) 0))
+                           (howm-reminder-search "!"))))
+    (mapc (lambda (item)
+            (howm-modify-in-background (lambda (item dummy)
+                                         (howm-view-open-item item))
+                                       #'howm-extend-deadline-here
+                                       nil nil item days))
+          hit)
+    (howm-menu-refresh-background)
+    (message "Extended %s deadline(s)." (length hit))))
+
+(defun howm-extend-deadline-here (item days)
+  (apply (lambda (form-reg cursor-reg) ;; use apply for destructuring-bind
+           (howm-modify-form #'howm-extend-deadline-doit
+                             form-reg cursor-reg days))
+         (let ((summary (howm-item-summary item)))
+           (string-match (howm-reminder-regexp ".") summary)
+           (mapcar (lambda (p)
+                     (concat (regexp-quote
+                              (substring summary (match-beginning p)))
+                             "$"))
+                   (list howm-reminder-regexp-date-pos
+                         howm-reminder-regexp-year-pos)))))
+
+(defun howm-extend-deadline-doit (days)
+  (or (looking-at howm-date-regexp)
+      (re-search-backward howm-date-regexp (line-beginning-position) t)
+      (error "Can't find corresponding date form."))
+  (howm-datestr-replace
+   (howm-datestr-shift (howm-time-to-datestr) 0 0 days)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; customize
 
@@ -797,19 +868,21 @@ or TODO is t."
          (reminder-menu (or schedule-menu todo-menu)))
     ;; Don't modify howm-reminder-marks.
     ;; Otherwise, defcustom will be confused for howm-reminder-menu-types, etc.
-    (mapcar* (lambda (var flag) (howm-modify-reminder-types var letter flag))
-             '(howm-reminder-types
-               howm-schedule-types howm-todo-types
-               howm-schedule-menu-types howm-todo-menu-types
-               howm-reminder-menu-types)
-             (list t schedule todo schedule-menu todo-menu reminder-menu))))
+    (howm-cl-mapcar* (lambda (var flag)
+                       (howm-modify-reminder-types var letter flag))
+                     '(howm-reminder-types
+                       howm-schedule-types howm-todo-types
+                       howm-schedule-menu-types howm-todo-menu-types
+                       howm-reminder-menu-types)
+                     (list t schedule todo
+                           schedule-menu todo-menu reminder-menu))))
 
 (defun howm-modify-reminder-types (var letter flag)
   "Modify variable VAR whose value is \"[...]\".
 Example:
-(setq foo \"[abc]\")
-(howm-modify-reminder-types 'foo \"d\" t)  foo ==> \"[abcd]\"
-(howm-modify-reminder-types 'foo \"b\" nil)  foo ==> \"[acd]\"
+ (setq foo \"[abc]\")
+ (howm-modify-reminder-types 'foo \"d\" t)  foo ==> \"[abcd]\"
+ (howm-modify-reminder-types 'foo \"b\" nil)  foo ==> \"[acd]\"
 "
   (let ((val (symbol-value var)))
     (when (not (string-match "^\\[\\(.*\\)\\]$" val))
@@ -842,9 +915,4 @@ Example:
 ;;                       #'howm-todo-priority-separator
 ;;                       'howm-reminder-separator-face t t)
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; provide
-
-(provide 'howm-reminder)
-
 ;;; howm-reminder.el ends here
