@@ -3,8 +3,6 @@
 ;;; @configure_input@
 
 (provide 'howm-version)
-(defconst howm-version "1.3.4")
-(defconst howm-en-dir "~endir~")
-(defconst howm-ja-dir "~jadir~")
+(defconst howm-version "1.4.1")
 
 ;;; howm-version.el ends here
