@@ -1,7 +1,7 @@
 ;;; riffle.el --- template of list browser with immediate preview
-;;; Copyright (c) 2004, 2005, 2006
-;;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;;; $Id: riffle.el,v 1.25 2006/01/06 15:35:46 hira Exp $
+;;; Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012
+;;;   HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
+;;; $Id: riffle.el,v 1.41 2011-12-31 15:07:29 hira Exp $
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -39,11 +39,8 @@
                 integer)
   :group 'howm-list-bufwin)
 (defcustom howm-view-split-horizontally nil
-  "If t, split window horizontally to show summary and contents.
-If nil, split vertically. Otherwise, split horizontally at only first time."
-  :type '(radio (const :tag "Vertically" nil)
-                (const :tag "Horizontally" t)
-                (const :tag "Horizontally at only first time" 1))
+  "If non-nil, split window horizontally to show summary and contents."
+  :type 'boolean
   :group 'howm-list-bufwin)
 (defcustom howm-view-keep-one-window nil
   "If nil, split windows automatically for summary and contents
@@ -51,13 +48,16 @@ even if you delete other windows explicitly."
   :type 'boolean
   :group 'howm-list-bufwin)
 (defcustom howm-view-pop-up-windows t
-  "If non-nil, Override `pop-up-windows'."
+  "If non-nil, override `pop-up-windows'."
   :type 'boolean
   :group 'howm-list-bufwin)
 
 ;; clean me: This value is copied to howm-view-open-recenter.
 (defvar howm-view-search-recenter 5)
 
+;; experimental [2008-05-23]
+(defvar riffle-keep-window nil)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; internal variables and accessors
 
@@ -110,6 +110,7 @@ even if you delete other windows explicitly."
 (defun riffle-type (&rest r)
   riffle-type)
 (defvar riffle-dispatchers (list #'riffle-type))
+(put 'riffle-dispatchers 'risky-local-variable t)
 
 (gfunc-with riffle-dispatchers
   (gfunc-def riffle-home (item))
@@ -130,6 +131,7 @@ even if you delete other windows explicitly."
   :group 'howm-hook)
 
 (defvar riffle-mode-map nil)
+(put 'riffle-mode-map 'risky-local-variable t)
 (defvar riffle-mode-syntax-table (make-syntax-table))
 (defvar riffle-mode-abbrev-table nil)
 
@@ -166,7 +168,8 @@ This function returns effective value of ITEM-LIST."
 (defun riffle-summary-subr (name item-list)
   (riffle-summary-set-mode)
   (riffle-summary-show item-list)
-  (riffle-summary-check t))
+  (unless riffle-keep-window
+    (riffle-summary-check t)))
 
 (defun riffle-summary-show (item-list)
   (buffer-disable-undo)
@@ -185,14 +188,15 @@ This function returns effective value of ITEM-LIST."
 (riffle-define-derived-mode riffle-summary-mode riffle-mode "RiffleS"
   "not yet"
   ;; make-local-hook is obsolete for emacs >= 21.1.
-  (make-local-hook 'post-command-hook)
+  (howm-funcall-if-defined (make-local-hook 'post-command-hook))
   (add-hook 'post-command-hook 'riffle-post-command t t)
   )
 
 (defun riffle-post-command ()
-  (if *riffle-summary-check*
-      (riffle-summary-check)
-    (setq *riffle-summary-check* t)))
+  (unless riffle-keep-window
+    (if *riffle-summary-check*
+        (riffle-summary-check)
+      (setq *riffle-summary-check* t))))
 
 (defun riffle-summary-current-item ()
   (let ((n (riffle-line-number)))
@@ -213,17 +217,13 @@ This function returns effective value of ITEM-LIST."
             (riffle-summary-update item force)))))))
 
 (defun riffle-summary-update (item &optional new)
-  (let ((execute t))
-    (when (one-window-p)
-      (cond (howm-view-keep-one-window (setq execute nil))
-            ((eq howm-view-split-horizontally t) (split-window-horizontally))))
-    (when execute
-      (riffle-summary-update-subr item new))))
+  (unless (and howm-view-keep-one-window (one-window-p))
+    (riffle-summary-update-subr item new)))
 (defun riffle-summary-update-subr (item &optional new)
   (let* ((*riffle-preview-p* t) ;; dirty
          (vbuf (riffle-contents-buffer new))
-	 (cwin (selected-window))
-	 (pop-up-windows (or pop-up-windows howm-view-pop-up-windows))
+         (cwin (selected-window))
+         (pop-up-windows (or pop-up-windows howm-view-pop-up-windows))
 ;;          (section (riffle-controller 'section item))
          (name (riffle-name))
          (type riffle-type)) ;; be careful to buffer local var.
@@ -259,8 +259,9 @@ This function returns effective value of ITEM-LIST."
 (defun riffle-summary-to-contents ()
   (interactive)
   (let ((b (current-buffer)))
-    (delete-other-windows)
-    (set-buffer b)
+    (unless riffle-keep-window
+      (delete-other-windows)
+      (set-buffer b))
     (let ((n (riffle-line-number)))
       (riffle-contents (riffle-name) (riffle-item-list) riffle-type
                        default-directory)
@@ -387,14 +388,23 @@ This function returns effective value of ITEM-LIST."
 
 (defun riffle-line-number (&optional pos)
   (save-excursion
-    (when pos
-      (goto-char pos))
-    (let ((raw (count-lines (point-min) (point))))
-      (if (bolp)
-          (+ raw 1)
-        raw))))
+    (save-restriction
+      (widen)
+      (when pos
+        (goto-char pos))
+      (let ((raw (count-lines (point-min) (point))))
+        (if (bolp)
+            (+ raw 1)
+          raw)))))
 
 (defun riffle-persistent-p (z)
+  "Return whether the buffer should be persistent or not.
+Note that the value of Z is funcall-ed if it is a function;
+consider to set `risky-local-variable' property.
+(cf)
+snap://Info-mode/elisp#File Local Variables
+snap://Info-mode/emacs#File Variables
+"
   (riffle-get-value z))
 
 (defun riffle-get-value (z)
@@ -409,10 +419,12 @@ This function returns effective value of ITEM-LIST."
 ;;   (message "%s -- %s" (buffer-name) (if (riffle-p) t nil)) ;; debug
   (if (riffle-p)
       (riffle-setup-window-configuration)
-    (delete-other-windows)))
+    (unless riffle-keep-window
+      (delete-other-windows))))
 
 (defvar riffle-window-initializer 'delete-other-windows)
 ;; (setq riffle-window-initializer '(lambda () (pop-to-buffer nil)))
+(put 'riffle-window-initializer 'risky-local-variable t)
 (defun riffle-setup-window-configuration ()
   (let ((orig (current-buffer))
         (s (riffle-summary-buffer))
@@ -439,12 +451,18 @@ This function returns effective value of ITEM-LIST."
 
 (defvar even-window-heights nil)  ;; xemacs doesn't have it.
 (defun riffle-pop-to-buffer (buf &optional size)
-  (when (one-window-p)
-    (split-window nil size howm-view-split-horizontally))
-  (let ((even-window-heights (if size
-                                 nil
-                               even-window-heights)))
-    (pop-to-buffer buf)))
+  (if riffle-keep-window
+      (switch-to-buffer buf)
+    (progn
+      (when (one-window-p)
+        (split-window nil size howm-view-split-horizontally))
+      (let ((even-window-heights (if size
+                                     nil
+                                   even-window-heights))
+            ;; Don't split windows further even when
+            ;; riffle-pop-to-buffer is called twice.
+            (pop-up-windows nil))
+        (pop-to-buffer buf)))))
 
 ;; 'Place' is line number at now
 (defun riffle-set-place (place)
@@ -452,10 +470,6 @@ This function returns effective value of ITEM-LIST."
 (defun riffle-get-place (&optional point)
   (riffle-line-number point))
 
-(defun riffle-set-mode-maybe (mode)
-  (when (not (eq major-mode mode))
-    (funcall mode)))
-
 ;; ;; needless?
 ;; (defun riffle-jump-to-summary ()
 ;;   (interactive)
